@@ -1,5 +1,6 @@
 use std::cell::Cell;
 use std::collections::{HashMap, VecDeque};
+use std::mem::MaybeUninit;
 use std::num::NonZeroUsize;
 use std::sync::{Arc, Condvar, Mutex};
 use std::time::Duration;
@@ -7,7 +8,9 @@ use std::{ptr, thread};
 
 use ignore_result::Ignore;
 
+use crate::task::mpsc::{self, Sender};
 use crate::task::{self, SchedFlow, Task};
+use crate::time;
 
 thread_local! {
     static SCHEDULER: Cell<Option<ptr::NonNull<Scheduler>>> = Cell::new(None);
@@ -45,20 +48,52 @@ impl Drop for Scope {
     }
 }
 
+/// Builder for [Runtime].
+#[derive(Default)]
+pub struct Builder {
+    parallelism: Option<usize>,
+}
+
+impl Builder {
+    /// Specifies the number of parallel threads for scheduling.
+    pub fn parallelism(&mut self, n: usize) -> &mut Self {
+        assert!(n > 0, "parallelism must not be zero");
+        self.parallelism = Some(n);
+        self
+    }
+
+    /// Constructs an [Runtime] to spawn and schedule tasks.
+    pub fn build(&mut self) -> Runtime {
+        let parallelism = self
+            .parallelism
+            .unwrap_or_else(|| thread::available_parallelism().unwrap_or(NonZeroUsize::new(4).unwrap()).get());
+        let (time_sender, time_receiver) = mpsc::unbounded(512);
+        let scheduler = Scheduler::new(parallelism, time_sender.clone());
+        let timer = time::Timer::new();
+        task::Builder::with_scheduler(&scheduler).spawn(move || {
+            time::timer(timer, time_receiver);
+        });
+        let ticker = thread::spawn(move || {
+            time::tick(time_sender);
+        });
+        let scheduling_threads = Scheduler::start(&scheduler);
+        Runtime { scheduler, ticker: MaybeUninit::new(ticker), scheduling_threads }
+    }
+}
+
 /// Runtime encapsulates io selecter, timer and task scheduler to serve spawned tasks.
 ///
 /// [Runtime::drop] will stop and join all serving threads.
 pub struct Runtime {
     scheduler: Arc<Scheduler>,
+    ticker: MaybeUninit<thread::JoinHandle<()>>,
     scheduling_threads: Vec<thread::JoinHandle<()>>,
 }
 
 impl Runtime {
     /// Constructs an runtime to serve spawned tasks.
     pub fn new() -> Runtime {
-        let scheduler = Scheduler::new();
-        let scheduling_threads = Scheduler::start(&scheduler);
-        Runtime { scheduler, scheduling_threads }
+        Builder::default().build()
     }
 
     /// Constructs a task builder to spawn task.
@@ -91,6 +126,8 @@ impl Drop for Runtime {
         for handle in self.scheduling_threads.drain(..) {
             handle.join().ignore();
         }
+        let ticker = unsafe { ptr::read(self.ticker.as_ptr()) };
+        ticker.join().ignore();
     }
 }
 
@@ -112,6 +149,7 @@ impl SchedulerState {
 
 pub(crate) struct Scheduler {
     parallelism: usize,
+    timer: Sender<time::Message>,
     state: Mutex<SchedulerState>,
     waker: Condvar,
 }
@@ -120,9 +158,8 @@ unsafe impl Send for Scheduler {}
 unsafe impl Sync for Scheduler {}
 
 impl Scheduler {
-    fn new() -> Arc<Scheduler> {
-        let parallelism = thread::available_parallelism().unwrap_or(NonZeroUsize::new(4).unwrap()).get();
-        Arc::new(Scheduler { parallelism, state: Mutex::new(SchedulerState::new()), waker: Condvar::new() })
+    fn new(parallelism: usize, timer: Sender<time::Message>) -> Arc<Scheduler> {
+        Arc::new(Scheduler { parallelism, timer, state: Mutex::new(SchedulerState::new()), waker: Condvar::new() })
     }
 
     /// Starts threads to serve spawned tasks.
@@ -150,6 +187,10 @@ impl Scheduler {
         SCHEDULER.with(|s| s.get().map(|s| unsafe { s.as_ref() }))
     }
 
+    pub(crate) fn try_time_sender() -> Option<Sender<time::Message>> {
+        Self::try_current().map(|s| s.timer.clone())
+    }
+
     pub fn sched(&self, t: Arc<Task>) {
         let mut state = self.state.lock().unwrap();
         let id = t.id();
@@ -192,6 +233,9 @@ impl Scheduler {
             return;
         }
         // This is the last scheduling thread.
+        drop(state);
+        self.timer.clone().send(time::Message::Stop).ignore();
+        state = self.state.lock().unwrap();
         while !state.registry.is_empty() {
             // SAFETY: Avoid compilation warning in read to `registry` and write to `runq`.
             let registry: &HashMap<u64, Arc<Task>> = unsafe { std::mem::transmute::<_, _>(&state.registry) };
@@ -211,3 +255,46 @@ impl Scheduler {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use pretty_assertions::assert_eq;
+
+    use super::*;
+
+    thread_local! {
+        static LOCAL_SECRET: Cell<usize> = Cell::new(0);
+    }
+
+    #[test]
+    #[should_panic]
+    fn runtime_builder_parallelism_zero() {
+        Builder::default().parallelism(0).build();
+    }
+
+    #[test]
+    fn runtime_builder_parallelism_one() {
+        let runtime = Builder::default().parallelism(1).build();
+        let secret = 333;
+        let set_secret = runtime.spawn(move || {
+            thread::sleep(Duration::from_secs(10));
+            LOCAL_SECRET.with(|cell| cell.set(secret));
+        });
+        let get_secret = runtime.spawn(move || LOCAL_SECRET.with(|cell| cell.get()));
+        set_secret.join().unwrap();
+        assert_eq!(secret, get_secret.join().unwrap());
+    }
+
+    #[test]
+    fn runtime_builder_parallelism_multiple() {
+        let runtime = Builder::default().parallelism(2).build();
+        let secret = 111;
+        let set_secret = runtime.spawn(move || {
+            thread::sleep(Duration::from_secs(10));
+            LOCAL_SECRET.with(|cell| cell.set(secret));
+        });
+        let get_secret = runtime.spawn(move || LOCAL_SECRET.with(|cell| cell.get()));
+        set_secret.join().unwrap();
+        assert_ne!(secret, get_secret.join().unwrap());
+    }
+}
